@@ -14,7 +14,6 @@
 #include "soup-auth.h"
 #include "soup.h"
 #include "soup-connection-auth.h"
-#include "soup-marshal.h"
 
 /**
  * SECTION:soup-auth
@@ -81,9 +80,11 @@ soup_auth_set_property (GObject *object, guint prop_id,
 
 	switch (prop_id) {
 	case PROP_REALM:
+		g_free (auth->realm);
 		auth->realm = g_value_dup_string (value);
 		break;
 	case PROP_HOST:
+		g_free (priv->host);
 		priv->host = g_value_dup_string (value);
 		break;
 	case PROP_IS_FOR_PROXY:
@@ -220,8 +221,8 @@ soup_auth_class_init (SoupAuthClass *auth_class)
  * This is called by #SoupSession; you will normally not create auths
  * yourself.
  *
- * Return value: the new #SoupAuth, or %NULL if it could not be
- * created
+ * Return value: (nullable): the new #SoupAuth, or %NULL if it could
+ * not be created
  **/
 SoupAuth *
 soup_auth_new (GType type, SoupMessage *msg, const char *auth_header)
@@ -525,6 +526,11 @@ soup_auth_free_protection_space (SoupAuth *auth, GSList *space)
 	g_slist_free_full (space, g_free);
 }
 
+/**
+ * soup_auth_get_saved_users:
+ *
+ * Return value: (transfer full) (element-type utf8):
+ */
 GSList *
 soup_auth_get_saved_users (SoupAuth *auth)
 {
