@@ -5,10 +5,6 @@
  * Copyright (C) 2007 Novell, Inc.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <string.h>
 
 #include "soup-path-map.h"
@@ -62,7 +58,7 @@ void
 soup_path_map_free (SoupPathMap *map)
 {
 	SoupPathMapping *mappings = (SoupPathMapping *)map->mappings->data;
-	guint i;
+	int i;
 
 	for (i = 0; i < map->mappings->len; i++) {
 		g_free (mappings[i].path);
@@ -83,8 +79,7 @@ static gboolean
 mapping_lookup (SoupPathMap *map, const char *path, int *match, int *insert)
 {
 	SoupPathMapping *mappings = (SoupPathMapping *)map->mappings->data;
-	guint i;
-	int path_len;
+	int i, path_len;
 	gboolean exact = FALSE;
 
 	*match = -1;
