@@ -3,6 +3,7 @@
 #include "test-utils.h"
 
 SoupURI *uri;
+GTlsDatabase *null_tlsdb;
 
 static void
 do_properties_test_for_session (SoupSession *session)
@@ -36,7 +37,7 @@ do_async_properties_tests (void)
 
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
 	g_object_set (G_OBJECT (session),
-		      SOUP_SESSION_SSL_CA_FILE, "/dev/null",
+		      SOUP_SESSION_TLS_DATABASE, null_tlsdb,
 		      SOUP_SESSION_SSL_STRICT, FALSE,
 		      NULL);
 	do_properties_test_for_session (session);
@@ -52,7 +53,7 @@ do_sync_properties_tests (void)
 
 	session = soup_test_session_new (SOUP_TYPE_SESSION_SYNC, NULL);
 	g_object_set (G_OBJECT (session),
-		      SOUP_SESSION_SSL_CA_FILE, "/dev/null",
+		      SOUP_SESSION_TLS_DATABASE, null_tlsdb,
 		      SOUP_SESSION_SSL_STRICT, FALSE,
 		      NULL);
 	do_properties_test_for_session (session);
@@ -105,7 +106,7 @@ do_strictness_test (gconstpointer data)
 	}
 	if (!test->with_ca_list) {
 		g_object_set (G_OBJECT (session),
-			      SOUP_SESSION_SSL_CA_FILE, "/dev/null",
+			      SOUP_SESSION_TLS_DATABASE, null_tlsdb,
 			      NULL);
 	}
 
@@ -147,12 +148,19 @@ do_session_property_tests (void)
 	GTlsDatabase *tlsdb;
 	char *ca_file;
 	SoupSession *session;
+	GParamSpec *pspec;
 
 	g_test_bug ("673678");
 
 	SOUP_TEST_SKIP_IF_NO_TLS;
 
 	session = soup_session_async_new ();
+
+	/* Temporarily undeprecate SOUP_SESSION_SSL_CA_FILE to avoid warnings. */
+	pspec = g_object_class_find_property (g_type_class_peek (SOUP_TYPE_SESSION),
+					      SOUP_SESSION_SSL_CA_FILE);
+	pspec->flags &= ~G_PARAM_DEPRECATED;
+
 	g_signal_connect (session, "notify::ssl-use-system-ca-file",
 			  G_CALLBACK (property_changed), &use_system_changed);
 	g_signal_connect (session, "notify::tls-database",
@@ -233,6 +241,9 @@ do_session_property_tests (void)
 	g_assert_true (ca_file_changed);
 
 	soup_test_session_abort_unref (session);
+
+	/* Re-deprecate SOUP_SESSION_SSL_CA_FILE */
+	pspec->flags |= G_PARAM_DEPRECATED;
 }
 
 /* GTlsInteraction subclass for do_interaction_test */
@@ -420,6 +431,7 @@ main (int argc, char **argv)
 {
 	SoupServer *server;
 	int i, ret;
+	GError *error = NULL;
 
 	test_init (argc, argv, NULL);
 
@@ -427,6 +439,9 @@ main (int argc, char **argv)
 		server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
 		soup_server_add_handler (server, NULL, server_handler, NULL, NULL);
 		uri = soup_test_server_get_uri (server, "https", "127.0.0.1");
+
+		null_tlsdb = g_tls_file_database_new ("/dev/null", &error);
+		g_assert_no_error (error);
 	} else
 		uri = NULL;
 
@@ -446,6 +461,7 @@ main (int argc, char **argv)
 	if (tls_available) {
 		soup_uri_free (uri);
 		soup_test_server_quit_unref (server);
+		g_object_unref (null_tlsdb);
 	}
 
 	test_cleanup ();
